import React from 'react';
import { connect } from 'dva';
import styles from "./index.css";
import Square from "../../components/Square"


function calculateWinner(squares) {
  const lines = [
    [0, 1, 2],
    [3, 4, 5],
    [6, 7, 8],
    [0, 3, 6],
    [1, 4, 7],
    [2, 5, 8],
    [0, 4, 8],
    [2, 4, 6]
  ];
  for (let i = 0; i < lines.length; i++) {
    const [a, b, c] = lines[i];
    if (squares[a] && squares[a] === squares[b] && squares[a] === squares[c]) {
      return squares[a];
    }
  }
  return null;
}


class TicTacToe extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      history: [
        {
          squares: Array(9).fill(null)
        }
      ],
      stepNumber: 0,
      xIsNext: true
    };
  }

  nextStep(key) {
    const ticTacToe = this.props.ticTacToe;
    const squares = ticTacToe.history[ticTacToe.stepNumber].squares;
    if (calculateWinner(squares) || squares[key]) {
      return;
    }
    this.props.dispatch({
      type: "ticTacToe/nextStep",
      payload: key
    });
  }

  jumpTo(key) {
    this.props.dispatch({
      type: "ticTacToe/jumpTo",
      payload: key
    });
  }

  render() {
    const ticTacToe = this.props.ticTacToe;
    const xIsNext = ticTacToe.xIsNext;
    const history = ticTacToe.history;
    const current = history[ticTacToe.stepNumber];
    const winner = calculateWinner(current.squares);

    const moves = history.map((item, move) => {
      const desc = move ?
        'Go to move #' + move :
        'Go to game start';
      return (
        <li key={move}>
          <button onClick={() => this.jumpTo(move)}>{desc}</button>
        </li>
      );
    });

    const squares = current.squares;
    return (
      <div className={styles.game}>
        <div className={styles['game-board']}>
          {squares.map((item, key) => (
            <Square key={key} backgroundColor={ticTacToe.backgroundColor} value={item} onClick={() => this.nextStep(key)} />
          ))}
        </div>
        <div className={styles['game-info']}>
          <div>{
            winner ? "Winner: " + winner :
              "Next player: " + (xIsNext ? "X" : "O")
          }</div>
          <ol>{moves}</ol>
        </div>
      </div>
    )
  }

}

export default connect(({ ticTacToe }) => ({ ticTacToe }))(TicTacToe)
